Package: kutils
Type: Package
Title: Project Management Tools
Version: 1.73
Date: 2023-09-17
Encoding: UTF-8
Authors@R: c(person("Paul","Johnson",email="pauljohn@ku.edu",role = c("aut", "cre")),
		   person("Benjamin","Kite",email="bakite@ku.edu",role="aut"),
		   person("Charles","Redmon",email="redmon@ku.edu",role="aut"),
		   person("Jared","Harpole",email="jared.harpole@gmail.com", role="ctb"),
  		   person("Kenna","Whitley",role="ctb"),
		   person("Po-Yi","Chen",role="ctb"), 
                   person("Shadi", "Pirhosseinloo",role="ctb")) 
Maintainer: Paul Johnson <pauljohn@ku.edu>
Description: Tools for data importation, recoding, and inspection.
    There are functions to create new project
    folders, R code templates, create uniquely named output
    directories, and to quickly obtain a visual summary for each
    variable in a data frame.  The main feature here is the systematic
    implementation of the "variable key" framework for data
    importation and recoding.  We are eager to have community feedback
    about the variable key and the vignette about it. In version 1.7,
    the function 'semTable' is removed. It was deprecated since 1.67.
    That is provided in a separate package, 'semTable'.
License: GPL-2
Depends: R (>= 3.3.0)
Imports: stats, utils, methods, foreign, xtable, plyr, openxlsx, RUnit
Suggests: rockchalk
RoxygenNote: 7.2.3
LazyData: TRUE
NeedsCompilation: no
Packaged: 2023-09-17 14:13:02 UTC; pauljohn
Author: Paul Johnson [aut, cre],
  Benjamin Kite [aut],
  Charles Redmon [aut],
  Jared Harpole [ctb],
  Kenna Whitley [ctb],
  Po-Yi Chen [ctb],
  Shadi Pirhosseinloo [ctb]
Repository: CRAN
Date/Publication: 2023-09-17 15:50:02 UTC
