# Generated by roxygen2: do not edit by hand

S3method(all.equal,key)
S3method(all.equal,keylong)
S3method(print,keyDiff)
S3method(print,keycheck)
S3method(print,likert)
export(alphaOnly)
export(assignMissing)
export(assignRecode)
export(checkCoercion)
export(cleanDataFrame)
export(colnamesReplace)
export(deduper)
export(deleteBogusColumns)
export(deleteBogusRows)
export(dev.create)
export(dir.create.unique)
export(dms)
export(dts)
export(escape)
export(file.backup)
export(importQualtrics)
export(initProject)
export(is.data.frame.simple)
export(isNA)
export(keyApply)
export(keyCheck)
export(keyDiff)
export(keyImport)
export(keyLookup)
export(keyRead)
export(keySave)
export(keyTemplate)
export(keyTemplateSPSS)
export(keyTemplateStata)
export(keyUpdate)
export(keysPool)
export(keysPoolCheck)
export(likert)
export(long2wide)
export(mergeCheck)
export(mgsub)
export(modifyVector)
export(n2NA)
export(padW0)
export(peek)
export(qualtricsBlockStack)
export(removeMatches)
export(reverse)
export(safeInteger)
export(shorten)
export(starsig)
export(stringbreak)
export(truncsmart)
export(updatePackages)
export(varlabTemplate)
export(wide2long)
export(writeCSV)
export(zapspace)
import(datasets)
importFrom(foreign,read.dta)
importFrom(foreign,read.spss)
importFrom(grDevices,dev.off)
importFrom(grDevices,devAskNewPage)
importFrom(grDevices,pdf)
importFrom(graphics,barplot.default)
importFrom(graphics,hist.default)
importFrom(graphics,par)
importFrom(graphics,strwidth)
importFrom(methods,as)
importFrom(openxlsx,addWorksheet)
importFrom(openxlsx,getSheetNames)
importFrom(openxlsx,read.xlsx)
importFrom(openxlsx,readWorkbook)
importFrom(openxlsx,saveWorkbook)
importFrom(openxlsx,writeDataTable)
importFrom(plyr,mapvalues)
importFrom(plyr,rbind.fill)
importFrom(stats,na.omit)
importFrom(stats,reshape)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(tools,package_dependencies)
importFrom(utils,available.packages)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,installed.packages)
importFrom(utils,modifyList)
importFrom(utils,packageDescription)
importFrom(utils,read.csv)
importFrom(utils,type.convert)
importFrom(utils,update.packages)
importFrom(utils,write.csv)
importFrom(utils,write.table)
importFrom(xtable,print.xtable)
importFrom(xtable,xtable)
